/*
 * JBox2D - A Java Port of Erin Catto's Box2D
 * 
 * JBox2D homepage: http://jbox2d.sourceforge.net/
 * Box2D homepage: http://www.box2d.org
 * 
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the authors be held liable for any damages
 * arising from the use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must not
 * claim that you wrote the original software. If you use this software
 * in a product, an acknowledgment in the product documentation would be
 * appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 * misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 */

package org.jbox2d.collision.shapes;

import org.jbox2d.common.Vec2;

/**
 * A circle shape definition.  Set the radius and localPosition
 * ( (0,0) by default ), and then call Body.createShape(ShapeDef)
 * to add the shape to a body.
 */
public class CircleDef extends ShapeDef {

	public float radius;
	public Vec2 localPosition;

	/**
	 * creates a circle definition at (0,0) with a radius of 1.
	 */
	public CircleDef() {
		type = ShapeType.CIRCLE_SHAPE;
		localPosition = new Vec2(0.0f, 0.0f);
		radius = 1.0f;
	}

	/**
	 * creates the circle definition at the given point with
	 * a radius of 1
	 * @param argVec
	 */
	public CircleDef(final Vec2 argVec){
		localPosition = argVec.clone();
		type = ShapeType.CIRCLE_SHAPE;
		radius = 1.0f;
	}
}