/*
 * JBox2D - A Java Port of Erin Catto's Box2D
 * 
 * JBox2D homepage: http://jbox2d.sourceforge.net/ 
 * Box2D homepage: http://www.box2d.org
 * 
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the authors be held liable for any damages
 * arising from the use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must not
 * claim that you wrote the original software. If you use this software
 * in a product, an acknowledgment in the product documentation would be
 * appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 * misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 */

//Updated to rev 89 of b2ContactSolver.h/.cpp

package org.jbox2d.dynamics.contacts;

import org.jbox2d.collision.Manifold;
import org.jbox2d.common.Settings;
import org.jbox2d.common.Vec2;
import org.jbox2d.dynamics.Body;

// Updated to rev 139 of b2ContactSolver.h
public class ContactConstraint {
    public final ContactConstraintPoint points[];

    public final Vec2 normal;

    public Manifold manifold;

    public Body body1;

    public Body body2;

    public float friction;

    public float restitution;

    public int pointCount;

    public ContactConstraint() {
        points = new ContactConstraintPoint[Settings.maxManifoldPoints];
        for (int i = 0; i < Settings.maxManifoldPoints; i++) {
            points[i] = new ContactConstraintPoint();
        }
        pointCount = 0;
        normal = new Vec2();
        manifold = new Manifold();
    }
    
    public void set(final ContactConstraint cp){
    	pointCount = cp.pointCount;
    	normal.set(cp.normal);
    	manifold.set(cp.manifold);
    	friction = cp.friction;
    	restitution = cp.restitution;
    	body1 = cp.body1;
    	body2 = cp.body2;
    	for(int i=0; i<cp.pointCount; i++){
    		points[i].set(cp.points[i]);
    	}
    }
}
